; ----------------------------------------------------------------------------
; BMOW CONSOLE
; ----------------------------------------------------------------------------	
.segment "CONSOLE"
.incbin "bmow\bmow_console.o"

console_standard_init = $0335
console_write_char = $0481

; ----------------------------------------------------------------------------
; BMOW KEYBOARD
; ----------------------------------------------------------------------------	
.segment "KEYBOARD"
.incbin "bmow\bmow_keyboard.o"

keyboard_standard_init = $06b8
keyboard_handle_scan_code = $06df
KEYBOARD_ADR_KEY_BUFFER := $059f
KEYBOARD_ADR_KEY_READY := $059e
KEYBOARD_ADR_MODIFIER_BUFFER := $05a0
KEYBOARD_ADR_C_DOWN  := $05a7
KEYBOARD_ADR_LEFT_CONTROL_DOWN := $05a3
KEYBOARD_ADR_RIGHT_CONTROL_DOWN := $05a4
KEYBOARD_ADR_DATA := $3f12 ; bank 0
KEYBOARD_ADR_CONTROL := $3f13 ; bank 0

.segment "EXTRA"
MSBASIC_ADR_LAST_SECOND: .byte 0
MSBASIC_ADR_IDLE_SECONDS: .byte 0
MSBASIC_IDLE_TIMEOUT := 60

; ----------------------------------------------------------------------------
; INITCONSOLE
; ----------------------------------------------------------------------------	
INITCONSOLE:
	jsr console_standard_init
	jsr keyboard_standard_init
	cli
	rts
	
isr_init:		
	; Create a subroutine in bank 0, which will return PCBANK in the accumulator.
	.byte $EB, $00 ; bank #$00
	lda #$A3 ; LDA stack-relative
	sta $FFF8
	lda #3 ; stack offset to return address bank byte
	sta $FFF9
	lda #$6B ; RTL
	sta $FFFA
	
	; call the subroutine in bank 0, to get PCBANK in A
	.byte $22, $F8, $FF, $00 ; JSL $00FFF8
	sta $FFFC
		
	; create a wrapper routine in bank 0, which performs a long jump to the real ISR
	lda #$22 ; JSL
	sta $FFF9
	lda #<bmow_isr
	sta $FFFA
	lda #>bmow_isr
	sta $FFFB	
	; bank byte of real ISR's address is already at $00FFFC
	lda #$40 ; RTI
	sta $FFFD 
	
	; install wrapper routine's address in the interrupt vector
	lda #$F9
	sta $FFFE
	lda #$FF
	sta $FFFF	

	; set active data bank
	lda $FFFC
	.byte $FB ; bank A; use the program bank for all future 16-bit memory references
	
	rts

; --- Interrupt service routine	
bmow_isr:
	php
	.byte $8B ; phb
	pha
	.byte $5A ; phy
	.byte $DA ; phx
	
	
	; the keyboard is memory-mapped in bank $00
	.byte $EB, $00 ; bank #$00
	
	; was the interrupt triggered by the keyboard?
	lda KEYBOARD_ADR_CONTROL
	and #$01
	bne @notkeyboard
	lda KEYBOARD_ADR_DATA
	
	; the handler routines use the program bank
	pha
	lda $FFFC
	.byte $FB ; bank A
	pla
	
	jsr keyboard_handle_scan_code

@notkeyboard:
	.byte $FA ; plx
	.byte $7A ; ply
	pla
	.byte $AB ; plb
	plp
	
	; return to bank 0 wrapper
	.byte $6B ; rtl 
		
; ----------------------------------------------------------------------------
; MONRDKEY
; Wait for a keypress, return it in A. Does not echo the keypress.
; ----------------------------------------------------------------------------				
MONRDKEY:
	lda #0
	sta MSBASIC_ADR_IDLE_SECONDS
MONRDKEY_LOOP:	
	lda #1
	cmp KEYBOARD_ADR_KEY_READY
	beq PROCKEY
	
	.byte $AF, $20, $3F, $00 ; lda RTC_ADR_SECONDS
	cmp MSBASIC_ADR_LAST_SECOND
	beq MONRDKEY_LOOP
	sta MSBASIC_ADR_LAST_SECOND
	inc MSBASIC_ADR_IDLE_SECONDS
	lda MSBASIC_ADR_IDLE_SECONDS
	cmp #MSBASIC_IDLE_TIMEOUT
	bcc MONRDKEY_LOOP
	jmp EXITBASIC
	
PROCKEY:	
	lda KEYBOARD_ADR_KEY_BUFFER
	;schamberlin: TODO: use KEYBOARD_ADR_MODIFIER_BUFFER to set control bits as needed
	;ldy KEYBOARD_ADR_MODIFIER_BUFFER
	.byte $9C, <KEYBOARD_ADR_KEY_READY, >KEYBOARD_ADR_KEY_READY ; clr KEYBOARD_ADR_KEY_READY
	
	rts
		

; ----------------------------------------------------------------------------
; INLIN_BMOW
; Read a line of input, ending when RETURN is pressed.
; Stores the line at INPUTBUFFER.
; Handles backspace, and echoes keypresses (except RETURN).
; Limits line length to 255 characters.
; On exit, X = length of the line, not including RETURN.
; ----------------------------------------------------------------------------   	
INLIN_BMOW:
	ldx #$00
	
@readloop:	
	jsr MONRDKEY
	
	cmp #CR
	beq @done
	
	cmp #8 ; backspace?
	beq @backspace
	
	cpx #$ff ; end of line?
	beq @readloop
	
@storchar:
	sta INPUTBUFFER,x
	inx
	bne @echochar ; always
	
@backspace:	
	cpx #0 ; start of line?
	beq @readloop
	
	dex

@echochar:
	jsr MONCOUT		
	jmp @readloop
		
@done:
	rts		  
        
; ----------------------------------------------------------------------------
; MONCOUT
; ----------------------------------------------------------------------------        
MONCOUT:  
	.byte $DA ; phx
	pha

	jsr console_write_char
	
	pla
	.byte $FA ; plx
	
	rts
		
EXITBASIC:
	; JML $000100
	.byte $5C, $00, $01, $00

				   